@@ -7,7 +7,7 @@ from plottr.analyzer.fitters.fitter_base  import Fit, FitResult
 
 class T1_Decay(Fit):
     @staticmethod
-    def model(coordinates: np.ndarray, amp: float, tau: float) -> np.ndarray: # type: ignore[override]
+    def model(coordinates: np.ndarray, amp: float, tau: float) -> np.ndarray:
         """ amp * exp(-1.0 * x / tau)"""
         return amp * np.exp(-1.0 * coordinates / tau)
     @staticmethod
@@ -18,7 +18,7 @@ class T1_Decay(Fit):
 
 class T2_Ramsey(Fit):
     @staticmethod
-    def model(coordinates: np.ndarray, amp: float, tau: float, freq: float, phase: float) -> np.ndarray: # type: ignore[override]
+    def model(coordinates: np.ndarray, amp: float, tau: float, freq: float, phase: float) -> np.ndarray:
         """ amp * exp(-1.0 * x / tau) * sin(2 * PI * freq * x + phase) """
         return amp * np.exp(-1.0 * coordinates / tau) * \
                np.sin(2 * np.pi * freq * coordinates + phase)
