@@ -7,15 +7,16 @@ from collections import OrderedDict
 from copy import deepcopy
 from dataclasses import dataclass
 from enum import Enum, unique, auto
+from types import TracebackType
 from typing import Dict, List, Type, Tuple, Optional, Any, \
     OrderedDict as OrderedDictType, Union
-from types import TracebackType
 
 import numpy as np
 
 from .. import Signal, Flowchart, QtWidgets
 from ..data.datadict import DataDictBase, DataDict, MeshgridDataDict
 from ..node import Node, linearFlowchart
+from ..utils import LabeledOptions
 
 __author__ = 'Wolfgang Pfaff'
 __license__ = 'MIT'
@@ -125,7 +126,7 @@ class PlotWidget(QtWidgets.QWidget):
     Implement a child class for actual plotting.
     """
 
-    def __init__(self, parent: Optional[QtWidgets.QWidget] = None) -> None:
+    def __init__(self, parent: Optional[PlotWidgetContainer] = None) -> None:
         super().__init__(parent=parent)
 
         self.data: Optional[DataDictBase] = None
@@ -133,6 +134,15 @@ class PlotWidget(QtWidgets.QWidget):
         self.dataStructure: Optional[DataDictBase] = None
         self.dataShapes: Optional[Dict[str, Tuple[int, ...]]] = None
         self.dataLimits: Optional[Dict[str, Tuple[float, float]]] = None
+        self.dataChanges: Dict[str, bool] = {
+            'dataTypeChanged': False,
+            'dataStructureChanged': False,
+            'dataShapesChanged': False,
+            'dataLimitsChanged': False,
+        }
+
+    def updatePlot(self) -> None:
+        return None
 
     def setData(self, data: Optional[DataDictBase]) -> None:
         """Set data. Use this to trigger plotting.
@@ -140,7 +150,7 @@ class PlotWidget(QtWidgets.QWidget):
         :param data: data to be plotted.
         """
         self.data = data
-        changes = self.analyzeData(data)
+        self.dataChanges = self.analyzeData(data)
 
     def analyzeData(self, data: Optional[DataDictBase]) -> Dict[str, bool]:
         """checks data and compares with previous properties.
@@ -239,21 +249,20 @@ class PlotDataType(Enum):
     grid2d = auto()
 
 
-@unique
-class ComplexRepresentation(Enum):
+class ComplexRepresentation(LabeledOptions):
     """Options for plotting complex-valued data."""
 
     #: only real
-    real = auto()
+    real = "Real"
 
     #: real and imaginary
-    realAndImag = auto()
+    realAndImag = "Real/Imag"
 
     #: real and imaginary, separated
-    realAndImagSeparate = auto()
+    realAndImagSeparate = "Real/Imag (split)"
 
     #: magnitude and phase
-    magAndPhase = auto()
+    magAndPhase = "Mag/Phase"
 
 
 def determinePlotDataType(data: Optional[DataDictBase]) -> PlotDataType:
@@ -369,9 +378,15 @@ class AutoFigureMaker:
         #: ids of all main plot items (does not contain derived/secondary plot items)
         self.plotIds: List = []
 
+        #: ids of all plot items, incl those who are 'joined' with 'main' plot items.
+        self.allPlotIds: List = []
+
         #: how to represent complex data.
         #: must be set before adding data to the plot to have an effect.
-        self.complexRepresentation = ComplexRepresentation.realAndImag
+        self.complexRepresentation: ComplexRepresentation = ComplexRepresentation.realAndImag
+
+        #: whether to combine 1D traces into one plot
+        self.combineTraces: bool = False
 
     def __enter__(self) -> "AutoFigureMaker":
         return self
@@ -536,11 +551,17 @@ class AutoFigureMaker:
         :param data: data arrays describing the plot (one or more independents, one dependent)
         :param join: ID of a plot item the new item should be shown together with in the same subplot
         :param labels: list of labels for the data arrays
-        :param plotDataType: what kind of plot data the supplied data contains (not needed, typically)
+        :param plotDataType: what kind of plot data the supplied data contains.
         :param plotOptions: options (as kwargs) to be passed to the actual plot functions (depends on the backend)
         :return: ID of the new plot item.
         """
 
+        if self.combineTraces and join is None:
+            prev = self.previousPlotId()
+            if prev is not None:
+                if len(data) == 2 and len(self.plotItems[prev].data) == 2:
+                    join = prev
+
         id = _generate_auto_dict_key(self.plotItems)
 
         # TODO: allow any negative number
@@ -556,13 +577,15 @@ class AutoFigureMaker:
 
         if labels is None:
             labels = [''] * len(data)
+        elif len(labels) < len(data):
+            labels += [''] * (len(data) - len(labels))
 
         plotItem = PlotItem(list(data), id, subPlotId,
                             plotDataType, labels, plotOptions)
 
         for p in self._splitComplexData(plotItem):
             self.plotItems[p.id] = p
-
+            self.allPlotIds.append(p.id)
         self.plotIds.append(id)
         return id
 
@@ -570,11 +593,47 @@ class AutoFigureMaker:
         """Get the ID of the most recently added plot item.
         :return: the ID.
         """
+        if not len(self.plotIds) > 0:
+            return None
+
         if len(self.plotIds) > 0:
             return self.plotIds[-1]
         else:
             return None
 
+    def findPlotIndexInSubPlot(self, plotId: int) -> int:
+        """find the index of a plot in its subplot
+
+        :param plotId: plot ID to check
+        :return: index at which the plot is located in its subplot.
+        """
+        if plotId not in self.allPlotIds:
+            raise ValueError("Plot ID not found.")
+
+        subPlotId = self.plotItems[plotId].subPlot
+        itemsInSubPlot = [i for i in self.allPlotIds if self.plotItems[i].subPlot == subPlotId]
+        return itemsInSubPlot.index(plotId)
+
+    def plotIdsInSubPlot(self, subPlotId: int) -> List[int]:
+        """return all plot IDs in a given subplot
+
+        :param subPlotId: ID of the subplot
+        :return: list of plot IDs
+        """
+        itemsInSubPlot = [i for i in self.allPlotIds if self.plotItems[i].subPlot == subPlotId]
+        return itemsInSubPlot
+
+    def dataDimensionsInSubPlot(self, subPlotId: int) -> Dict[int, int]:
+        """Determine what the data dimensions are in a subplot.
+
+        :param subPlotId: ID of the subplot
+        :return: dictionary with plot id as key, data dimension (i.e., number of independents) as value.
+        """
+        ret: Dict[int, int] = {}
+        for plotId in self.plotIdsInSubPlot(subPlotId):
+            ret[plotId] = len(self.plotItems[plotId].data) - 1
+        return ret
+
     # Methods to be implemented by inheriting classes
     def makeSubPlots(self, nSubPlots: int) -> List[Any]:
         """Generate the subplots. Called after all data has been added.
